PROGRAM_NAME='switcher queue'
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

dvswitcher					=	5001:1:0
dvTp								=	10001:1:0
(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

conSwitcherCommandDepth    = 100       //max depth for switcher commands
conSwitcherQueueMax        = 40        //max # of commands stored in queue

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

volatile integer iSwitcherQueueTail        //var to hold oldest command index
volatile integer iSwitcherQueueHead        //var to hold newest command index
volatile char cSwitcherQueue[conSwitcherQueueMax][conSwitcherCommandDepth]    //queue to hold switcher commands
volatile integer iSwitcherQueueReady       //var to store ready status of switcher to rcv commands
volatile integer iSwitcherQueueHasItems    //var to store whether or not commands are in queue

volatile integer iInputSelect[]=
{
	100,101,102,103,104,105,106,107,108,
	200,201,202,203,204,205,206,207,208,
	300,301,302,303,304,305,306,307,308,
	400,401,402,403,404,405,406,407,408,
	500,501,502,503,504,505,506,507,508,
	600,601,602,603,604,605,606,607,608,
	700,701,702,703,704,705,706,707,708,
	800,801,802,803,804,805,806,807,808
}

define_call 'add to switcher queue' (char cCommand[conSwitcherCommandDepth])
{
	select
	{
		active(iSwitcherQueueHead >= conSwitcherQueueMax):    //if switcher queue has reached limit
		{
			if(iSwitcherQueueTail <> 1)                       //do not overwrite if full, head at max and tail at start
			{
				iSwitcherQueueHead = 1                        //rollover
				cSwitcherQueue[iSwitcherQueueHead] = "cCommand"    //add new command to queue
				on[iSwitcherQueueHasItems]                    //turn on queue has items status
			}
		}
		active(iSwitcherQueueTail <> (iSwitcherQueueHead + 1)):   //do not overwrite if full, newest command would overwrite oldest command
		{
			iSwitcherQueueHead++                              //increment que head variable
			cSwitcherQueue[iSwitcherQueueHead] = "cCommand"   //add new command to queue
			on[iSwitcherQueueHasItems]                        //turn on queue has items status
		}
	}
}

define_call 'check switcher queue'
{
	if(iSwitcherQueueHasItems and iSwitcherQueueReady)        //if queue has items and switcher is ready to receive commands
	{
		off[iSwitcherQueueReady]                              //turn off ready status
		if(iSwitcherQueueTail >= conSwitcherQueueMax)           //if the switcher queue tail is equal to the max
		{
			iSwitcherQueueTail = 1                            //rollover
		}                                                               
		else                                                  //if switcher queue tail is not equal to max
		{
			iSwitcherQueueTail++                              //increment queue tail variable
		}                                                                                  
		if(iSwitcherQueueTail = iSwitcherQueueHead)               //if command sending caught up
		{
			off[iSwitcherQueueHasItems]                         //turn off queue has items status
		}                                                                                      
		send_string dvSwitcher,"(cSwitcherQueue[iSwitcherQueueTail])"    //send next command to switcher
		wait 1                                                          //wait for 1/10th of a second
		{
			on[iSwitcherQueueReady]                                     //turn ready status back on
		}
	}
}

(***********************************************************)
(*                THE EVENTS GO BELOW                    *)
(***********************************************************)
DEFINE_EVENT

data_event[dvswitcher]
{
	online:
	{
		send_command data.device,"'SET BAUD 9600,N,8,1 485 DISABLED'"
		on[iSwitcherQueueReady]//turn on queue ready status
	}
}

button_event[dvTp,iInputSelect]
{
	push:
	{
		local_var integer input 	//var to store input selected 
		local_var integer output 	 //var to store output selected
		output = (button.input.channel / 100)	//output is equal to the button channel number                                                      //divided by 100
		input = (button.input.channel % 100)	//input is equal to the remainder from the button
		call 'add to switcher queue' ("$01,(80+itohex(input)),(80+itohex(output)),$81")
	}
}

DEFINE_PROGRAM

call 'check switcher queue'